/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Channel.hpp
 *
 *  Created on: 2010/11/11
 *      Author: HARK Development Team
 */

#ifndef CHANNEL_HPP_
#define CHANNEL_HPP_

#include <Logger.hpp>
#include <Utils.hpp>

namespace NSHarkBinaural
{

/**
 * 信号のチャンネルを表す定数を集めた構造体
 */
struct Channel
{
    /**
     * 信号のチャンネルの種類を表す列挙型
     */
    typedef enum
    {
        /**
         * 右チャンネル
         */
        R,

        /**
         * 左チャンネル
         */
        L
    } Kind;
};

/**
 * 任意のデータの組を保持するクラス.
 * 2つ1組のデータに、左、右と名前をつけて保持する
 */
template <typename TLeft, typename TRight>
class ChannelHolder
{
    /**
     * TLeftの参照型.
     * 元々TLeftが参照型ならTLeftとTLeftReferenceは同じ。
     * 元々TLeftが非参照型ならTLeft&とTLeftReferenceが同じ。
     */
    typedef typename AddReference<TLeft>::type TLeftReference;

    /**
     * TRightの参照型.
     * 元々TRightが参照型ならTRightとTRightReferenceは同じ。
     * 元々TRightが非参照型ならTRight&とTRightReferenceが同じ。
     */
    typedef typename AddReference<TRight>::type TRightReference;

    /**
     * TLeftのconstな参照型.
     */
    typedef typename AddConst<TLeftReference>::type TLeftConstReference;

    /**
     * TRightのconstな参照型.
     */
    typedef typename AddConst<TRightReference>::type TRightConstReference;
public:
    /**
     * スマートポインタ型
     */
    typedef boost::shared_ptr<ChannelHolder> ptr_t;

    /**
     * クラスを生成する.
     * @param left 「左」に関係するデータ
     * @param right 「右」に関係するデータ
     */
    ChannelHolder(TLeftReference left, TRightReference right)
        : left(left), right(right), logger(Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME))
    {
        // 保持するデータの受け渡しは参照型で行いたいので、AddReferenceで参照型を生成して使う
        logger.Finest("ChannelHolder::ChannelHolder");
    }

    /**
     * クラスを破棄する.
     */
    ~ChannelHolder()
    {
        logger.Finest("ChannelHolder::~ChannelHolder");
    }

    /**
     * 左のデータを取得する.
     * @return 左のデータ
     */
    TLeftReference Left()
    {
        // 保持するデータの受け渡しは参照型で行いたいので、AddReferenceで参照型を生成して使う
        return left;
    }

    /**
     * 左のデータを取得する.
     * @return 左のデータ
     */
    TLeftConstReference Left() const
    {
        // 保持するデータの受け渡しは参照型で行いたいので、AddReferenceで参照型を生成して使う
        return left;
    }

    /**
     * 右のデータを取得する.
     * @return 右のデータ
     */
    TRightReference Right()
    {
        // 保持するデータの受け渡しは参照型で行いたいので、AddReferenceで参照型を生成して使う
        return right;
    }

    /**
     * 右のデータを取得する.
     * @return 右のデータ
     */
    TRightConstReference Right() const
    {
        // 保持するデータの受け渡しは参照型で行いたいので、AddReferenceで参照型を生成して使う
        return right;
    }

private:
    /**
     * 左のデータ.
     * データの受け渡しには参照型を使うが、データの保持には指定された型をそのまま使う。
     * 指定された型（TLeft）が参照型か非参照型かは関係ない。
     */
    TLeft left;

    /**
     * 右のデータ.
     * データの受け渡しには参照型を使うが、データの保持には指定された型をそのまま使う。
     * 指定された型（TRight）が参照型か非参照型かは関係ない。
     */
    TRight right;

    Logger& logger;
};

}

#endif /* CHANNEL_HPP_ */
