/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * EventHistoryManager.hpp
 *
 *  Created on: 2011/01/24
 *      Author: HARK Development Team
 */

#ifndef EVENTHISTORYMANAGER_HPP_
#define EVENTHISTORYMANAGER_HPP_

#include <vector>
#include <list>
#include <boost/shared_ptr.hpp>
#include <boost/tuple/tuple.hpp>
#include <DimensionPromise/types/Angle.hpp>
#include <HarmonicSound.hpp>
#include <Band.hpp>
#include <EventID.hpp>
#include <EventHistory.hpp>
#include <Logger.hpp>

namespace NSHarkBinaural
{

class EventHistoryManager
{
public:
    typedef boost::shared_ptr<HarmonicSound> HarmonicSoundPtr;
    typedef std::vector<HarmonicSoundPtr> HarmonicSoundVector;
    typedef boost::shared_ptr<HarmonicSoundVector> HarmonicSoundVectorPtr;

    typedef boost::tuple<EventID, Band, NSDimensionPromise::Angle> UpdateInfo;

    EventHistoryManager();

    /**
     * ヒストリーを調べ、soundsの要素にイベントIDを割り振る.
     */
    void CheckSoundHistory(HarmonicSoundVectorPtr sounds, const timeval& currentTime);

    void CheckSoundHistory(boost::shared_ptr<std::vector<NSDimensionPromise::Angle> > directions, const timeval& currentTime);


    /**
     * 有効なPAUSE期間を設定する.
     * デフォルト値は1秒
     */
    void SetValidPausePeriod(const timeval& validPausePeriod)
    {
        this->validPausePeriod = validPausePeriod;
    }

    /**
     * 近い音だと判断する角度の範囲を設定する.
     * 実際には、この角度の2倍の角度に含まれる音は近いと判断する
     */
    void SetNearDirectionThreshold(NSDimensionPromise::Angle nearDirectionThreshold)
    {
        this->nearDirectionThreshold = nearDirectionThreshold;
    }

    const std::list<EventHistory>& GetAllEvents() const
    {
        return this->eventHistories;
    }

    void SetEventUpdateRatioThreshold(double threshold)
    {
        this->eventUpdateRatioThreshold = threshold;
    }

private:
    Logger& logger;

    /// イベントヒストリー一覧（有効なイベントのみ格納している）
    std::list<EventHistory> eventHistories;

    /// 二つの方向が近いかどうかを決める閾値.
    NSDimensionPromise::Angle nearDirectionThreshold;

    /// PAUSE状態が続いても、一つの有効なイベントだとみなす期間.
    timeval validPausePeriod;

    /// 現在の最大イベントID
    int headid;

    /// イベントの更新頻度の閾値（これ以下になるとイベント終了）
    double eventUpdateRatioThreshold;

    void Update(std::vector<UpdateInfo>& updateInfoList, const timeval& currentTime);

    /**
     * 与えられた基音と方向に一番近いイベントを見つける.
     */
    EventID FindNearestEvent(Band fundamental, NSDimensionPromise::Angle direction, const timeval& currentTime);

    /**
     * 二つの方向が十分近いかどうか.
     */
    bool IsNear(NSDimensionPromise::Angle dir1, NSDimensionPromise::Angle dir2);

    /**
     * 既に終了したイベントをリストから消去する.
     */
    void EraseOutOfDateEvents();

    /**
     * イベントIDが割り当てられていないものに対し、新しいイベントを生成し割り当てる.
     * updateInfoListの中で、get<0>(updateInfoList[i]).IsConnected() == false
     * であるi番目の要素に、新しくイベントを生成して代入する。
     */
    void MakeNewEvents(std::vector<UpdateInfo>& updateInfoList, const timeval& currentTime);

    /**
     * 新しいイベントを生成し、管理リストに追加する.
     */
    EventID MakeNewEvent(const timeval& currentTime, Band fundamental, NSDimensionPromise::Angle direction);

    /**
     * 次の未使用のイベントIDを生成する.
     */
    EventID NextEID();
};

}

#endif /* EVENTHISTORYMANAGER_HPP_ */
